@@ -23,10 +23,12 @@ combML_bs=function(y_tr_lt,z_tr_lt,y_tr_rt,z_tr_rt,
       mse_va_lt[r]=sum((y_va_lt_num-y_hat_lt)^2)
     }
   }
+  if(sum(!is.na(mse_va_lt))==0)
+    return(list(mse_va=NA,fit_lt=NA,fit_rt=NA,ML_lt=NA,ML_rt=NA))
 
-  #2. MLs for left child node
+  #2. MLs for right child node
   fit_rt=list()
-  mse_tr_rt=rep(NA,R)
+  #mse_tr_rt=rep(NA,R)
   mse_va_rt=rep(NA,R)
   for(r in 1:R){
     fit_rt_r=FUN1[[r]](y_tr_rt,z_tr_rt)
@@ -39,27 +41,25 @@ combML_bs=function(y_tr_lt,z_tr_lt,y_tr_rt,z_tr_rt,
       mse_va_rt[r]=sum((y_va_rt_num-y_hat_rt)^2)
     }
   }
-
-  #3. Choose the best ML comb
-  #mse_tr=outer(mse_tr_lt,mse_tr_rt,"+")/n_tr
-  mse_va=outer(mse_va_lt,mse_va_rt,"+")/n_va
-  if(sum(!is.na(mse_va))==0)
+  if(sum(!is.na(mse_va_rt))==0)
     return(list(mse_va=NA,fit_lt=NA,fit_rt=NA,ML_lt=NA,ML_rt=NA))
 
-  ab_hat=which(mse_va==min(mse_va,na.rm=TRUE), arr.ind = TRUE)
-  if(nrow(ab_hat)>1) #if there are ties, randomly select
-    ab_hat=ab_hat[sample(1:nrow(ab_hat),1),]
-
-  a_hat=ab_hat[1]
-  b_hat=ab_hat[2]
-
+  #3. Choose the best ML comb
+  mse_va_lt_min=min(mse_va_lt,na.rm=TRUE)
+  a_hat=which(mse_va_lt==mse_va_lt_min)
+  if(length(a_hat)>=2)
+    a_hat=sample(a_hat,1)
   fit_lt=fit_lt[[a_hat]]
-  fit_rt=fit_rt[[b_hat]]
-
-  mse_va=mse_va[a_hat,b_hat]
-
   ML_lt=MLlist[a_hat]
+
+  mse_va_rt_min=min(mse_va_rt,na.rm=TRUE)
+  b_hat=which(mse_va_rt==mse_va_rt_min)
+  if(length(b_hat)>=2)
+    b_hat=sample(b_hat,1)
+  fit_rt=fit_rt[[b_hat]]
   ML_rt=MLlist[b_hat]
 
+  mse_va=(mse_va_lt_min+mse_va_rt_min)/n_va
+
   return(list(mse_va=mse_va,fit_lt=fit_lt,fit_rt=fit_rt,ML_lt=ML_lt,ML_rt=ML_rt))
 }
